%***************************
% Written by Farinaz Edalat
% May 9, 2005
%***************************

%************************************************************
% Description: Find the symbol-error-probability and BER of M-QAM and BPSK
% modulations for ranges of SNR values and plot the results
% NOTE: the formula to find M-QAM error probability is taken from 
% "Communication Systems Engineering" by Proakis, and is for rectangular
% M-QAMs, i.e. M=2^(even) so that an M-QAM can be decomposed to 2
% sqrt(M)-PAMs. 
% In addition, all the modulations {BPSK, and M-QAMs} have same 
% maximum symbol energy = 2.

%************************************************************

% Given Parameters**********
 M = [4,16,64,256];   %M-QAM values
 dmin = [2, 2/3, 2/7, 2/15];
%**************************** 
 
 Eavg = dmin.^2 .* (M-1) / 6;
 Eavg_db = 10*log10(Eavg);
 E_bpsk = 2;
 
% Calculate Symbol Error Probabilities 
pe_qam = [];
p_bpsk = [];
for (snr_db = 0:0.1:35)
    snr = 10.^(snr_db/10);
    pe_pam = 2*(1-1./sqrt(M)).*Q(sqrt(3*snr./(M-1)));
    pcorrect = (1 - pe_pam).^2;
    pe_qam = [pe_qam ; (1-pcorrect)];
    p_bpsk = [p_bpsk Q(sqrt(2*snr))];
end


ber_bpsk = p_bpsk;
ber_2 = pe_qam(:,1)/2;
ber_4 = pe_qam(:,2)/4;
ber_6 = pe_qam(:,3)/6;
ber_8 = pe_qam(:,4)/8;

% Plot BER versus normalized SNR by transmitted symbol energy
snr_db = [0:0.1:35];
figure(1)
semilogy(snr_db-Eavg_db(1), ber_bpsk,'k-',snr_db-Eavg_db(1), ber_2,'r-',snr_db-Eavg_db(2), ber_4,'b-',snr_db-Eavg(3), ber_6,'g-',snr_db-Eavg(4), ber_8,'m-')
grid on;
xlabel('SNR (dB) (normalized by transmitted symbol energy)')
ylabel('BER')
legend('BPSK', '4-QAM', '16-QAM', '64-QAM', '256-QAM')
axis([0 32 1e-5 1e-1])

% Plot BER versus SNR per bit
% SNR-per-bit=(Symbol average energy-per-bit)/Noise variance
snr_db = [0:0.1:35];
figure(2)
semilogy(snr_db-10*log10(1), ber_bpsk,'k-',snr_db-10*log10(2), ber_2,'r-',snr_db-10*log10(4), ber_4,'b-',snr_db-10*log10(6), ber_6,'g-',snr_db-10*log10(8), ber_8,'m-')
grid on;
xlabel('SNR per bit (dB) (not-normalized)')
ylabel('BER')
legend('BPSK', '4-QAM', '16-QAM', '64-QAM', '256-QAM')
axis([0 32 1e-5 1e-1])



