%***************************
% Written by Farinaz Edalat
% May 9, 2005
%***************************

%********************************************************************
% Description: For all N subcarriers,
% (1) Calculate the symbol average energy at the input of IFFT (E_X)
% (2) Calculate the estimated channel (freq-response) energy |H^2| (= E_H)
% (3) Calculate the Noise in each subchannel 
%       at the output of FFT Y=HX+Noise --> Noise = Y - HX
%       Next, find the Noise average energy (=E_N)
% (4) Calculate the SNR

% Given Parameters:***************
x = loadsig_cppsim('test_rx.tr0'); 
N = 8; %num_channels
p = 1; %num_pilots
%*********************************

% (1) Transmitted symbols at the input of IFFT
k = 0;
Eavg = [];
store_i = [];
store_q = [];
for (n = 1:N)
    vec_pilot_i = sprintf('xi11_vec_pilot_i_%d',k); %this is vec_qam_i delayed 
    vec_pilot_q = sprintf('xi11_vec_pilot_q_%d',k);
    u_i = evalsig(x,vec_pilot_i);
    u_q = evalsig(x,vec_pilot_q);
    u_i = u_i(p+1:length(u_i));
    u_q = u_q(p+1:length(u_q));
   
    store_i = [store_i, u_i];
    store_q = [store_q, u_q];

    k = k+1;
    u = u_i + j*u_q;
    u_energy = sum(abs(u).^2)/length(u);  
    Eavg = [Eavg u_energy];
  %  scatterplot(u)    
end
E_X = Eavg
X_i = store_i;
X_q = store_q;
X = X_i + j*X_q; %X(:,i) are the transmitted symbols in the i-th subchannel


% (2) Channel response energy
load -ascii ch_estimator_h_mag.dat; 
load -ascii ch_estimator_h_phase.dat;
H_mag = ch_estimator_h_mag;
H_phase = ch_estimator_h_phase;
H = H_mag.*exp(j*H_phase);

E_H = (H_mag.^2).'

% (3) Noise and Average Noise Energy in each subchannel
k = 0;
Eavg = [];
store_i = [];
store_q = [];
for (n = 1:N)
   
    vec_dqam_i = sprintf('xi11_vec_dqam_i_%d',k);
    vec_dqam_q = sprintf('xi11_vec_dqam_q_%d',k);
    u_i = evalsig(x,vec_dqam_i);
    u_q = evalsig(x,vec_dqam_q);
    u_i = u_i(p+1:length(u_i));
    u_q = u_q(p+1:length(u_q));
    
    store_i = [store_i, u_i];
    store_q = [store_q, u_q];
        
    k = k+1;
    u = u_i + j*u_q;
    u_energy = sum(abs(u).^2)/length(u);  
    Eavg = [Eavg u_energy];
   %scatterplot(u)    
end
E_dqam = Eavg;
Y_i = store_i;
Y_q = store_q;
Y = Y_i + j*Y_q;

store = [];
Eavg = [];
for (i=1:N)
    u = X(:,i) * H(i);
    store = [store, u];  
    u_energy = sum(abs(u).^2)/length(u);
    Eavg = [Eavg u_energy];
end
E_HX = Eavg; % E_HX= E_X .* E_H
HX = store;

Noise = Y - HX; 

Eavg = [];
for (i=1:N)
    u = Noise(:,i);
    u_energy = sum(abs(u.^2))/length(u);
    Eavg = [Eavg u_energy];
end
E_N = Eavg 

% (4) Calculate the normalized SNR = E[H^2]/E[N^2]
snr = 10*log10(E_H ./ E_N)

